/*
isd300_udma V0.9.2

Code to enable UDMA in config byte or attempt I2C EEPROM write for the 
ISD-300A / ISD-300LP

Requires libusb 0.1 
Tested with 0.1.12.1 win32 variant
May require usb_detach_kernel_driver_np before usb_claim_interface
if ported to linux.

(c) 2009 John Dobbler
With contibutions to EEPROM writing from Boris Gjenero 

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 * KIND, either express or implied.
 *
*/

#include <stdio.h>
#include "usb.h"
#include "isd300_udma_id.h"

/* Uncomment to display read data */
//#define DISPLAY_PARAMS

#define UDMA_BYTE      0x0c
#define NUM_PARAMS     0x10
#define UDMA_EN_BITS   0x18
#define UDMA_MODE_MASK 0x07
#define UDMA_EN_ATA    0x10
#define UDMA_EN_ATAPI  0x08
#define UDMA_DEFAULT_MODE 4

#define ISD300A_SIG0 0x54
#define ISD300A_SIG1 0x4d

#define ISD300LP_SIG0 0x4b
#define ISD300LP_SIG1 0x50

#define FIND_ERROR -1
#define FIND_OK     0
#define FIND_ABORT -2
#define TARGET_CONFIG 0
#define TARGET_EEPROM 2

int find_isd300_device(usb_dev_handle *udev, struct usb_device *dev)
{
    int i = 0, ret = FIND_ERROR;
    char target;
    unsigned char sig_byte[2];
        
    while(i < NUMBER_OF_USB_IDS)
    {
        /* Check for known USB IDs */
        if((dev->descriptor.idVendor==isd300_devices[i].idVendor)
            &&(dev->descriptor.idProduct==isd300_devices[i].idProduct))
        {
            usb_claim_interface(udev,0);            
            i = NUMBER_OF_USB_IDS;    // Stop searching
            /* Check we have the correct signature in I2C EEPROM*/
            usb_control_msg(udev, 0xc0, 0x02, 0x02, 0, &sig_byte[0], 2, 1000);
            if((sig_byte[0]==ISD300A_SIG0)&&(sig_byte[1]==ISD300A_SIG1))
            { 
                ret = FIND_OK;
                printf("Found ISD-300A\n");
            }
            else if((sig_byte[0]==ISD300LP_SIG0)&&(sig_byte[1]==ISD300LP_SIG1))
            { 
                ret = FIND_OK;
                printf("Found ISD-300LP\n");
            }
            else 
            {
                printf("Found compatible player, but problem with ISD-300\n");
                ret = FIND_ABORT;
            }
        }                                
        else i++;  
    }
    
    if(ret == FIND_OK)
    {
        printf("\nSelect target:\n");
        printf("1) Config bytes (temporary)\n");
        printf("2) EEPROM       (permanent)\n");
        printf("Any other key to exit\n\n");
        
        target = getch();

        if (target == '1')
        {
            printf("Config RAM selected\n");
            ret = TARGET_CONFIG;
        }
        else if (target == '2')
        {
            printf("EEPROM selected\n");
            ret = TARGET_EEPROM;
        }
        else
        {
            printf("Aborting\n");
            ret = FIND_ABORT;
        }
        
    }
    return ret;
}    

void enable_udma(usb_dev_handle *udev, unsigned char* usb_data, int target)
{
    char option;
    int num_check, udma_mode = UDMA_DEFAULT_MODE;
    unsigned char check_data;
    
    if((target!=TARGET_CONFIG)&&(target!=TARGET_EEPROM)) return;
    
    printf("ATA UDMA enable is ");
    if (usb_data[UDMA_BYTE] & UDMA_EN_ATA) printf("on\n");
    else printf("off\n");

    printf("ATAPI UDMA enable is ");
    if (usb_data[UDMA_BYTE] & UDMA_EN_ATAPI) printf("on\n");
    else printf("off\n");

    printf("Max UDMA mode is %d\n\n", usb_data[UDMA_BYTE] & UDMA_MODE_MASK);

    /* Enable ATA and ATAPI UDMA (in copy of data)*/
    usb_data[UDMA_BYTE] |= UDMA_EN_BITS;            
    
    do
    {
        /* Set mode (in copy of data)*/
        usb_data[UDMA_BYTE] &= ~UDMA_MODE_MASK;
        usb_data[UDMA_BYTE] |= udma_mode & UDMA_MODE_MASK;
        
        printf("Local copy of config byte 0xc now 0x%x\n", usb_data[UDMA_BYTE]);
        printf("(UDMA enabled for ATA and ATAPI at mode %d if available)\n",udma_mode);
        printf("Write to ");
        if (target == TARGET_CONFIG) printf("config RAM");
        else printf("EEPROM");
        printf(" (y/n)?  0-4 will change UDMA mode\n");
                
        option = getch();
        num_check = ((int)option) - 0x30; // Subtract start of ascii numbers
        if((num_check >= 0) && (num_check <= 4)) udma_mode = num_check;
        
    } while (num_check==udma_mode);

    if((option=='y')||(option=='Y'))
    {
        if(target == TARGET_CONFIG)
        {
            usb_control_msg(udev, 0x40, 0x01, TARGET_CONFIG, UDMA_BYTE, &usb_data[UDMA_BYTE], 0x01, 0);
        }
        else if (target == TARGET_EEPROM)
        {
            usb_control_msg(udev, 0x40, 0x01, TARGET_EEPROM, 0, &usb_data[0], NUM_PARAMS, 10000);
        }
        usb_control_msg(udev, 0xc0, 0x02, target, UDMA_BYTE, &check_data, 1, 1000);
        if(check_data==usb_data[UDMA_BYTE])
        {
            printf("Write succeeded, now resetting USB\n");
            /* Need to reset for settings to take effect */
            usb_reset(udev);
        }               
        else
        {
            printf("Write failed\n");
            if(target == TARGET_EEPROM) printf("The EEPROM hardware write protect may be enabled\n");  
        }
    }        
}
                

int find_and_modify_isd300(struct usb_device *dev)
{
    usb_dev_handle *udev;
    struct usb_device *sub_dev;
    int i, j, target, ret=FIND_ERROR;
    unsigned char usb_data[NUM_PARAMS];

    for (i = 0; i < dev->num_children; i++)
    {
        sub_dev = dev->children[i];
        udev = usb_open(sub_dev);
        if (udev) 
        { 
            target = find_isd300_device(udev, sub_dev);
            if (target != FIND_ERROR)
            {
                if(target==FIND_ABORT) ret=FIND_ABORT;
                else
                {
                    ret = FIND_OK;
                    printf("Reading existing parameters:\n");
                    /* Read and current config bytes */
                    usb_control_msg(udev, 0xc0, 0x02, target, 0, &usb_data[0], NUM_PARAMS, 1000);
                    #ifdef DISPLAY_PARAMS
                        for(j=0;j<NUM_PARAMS;j++) printf("0x%x = 0x%x\n", j, usb_data[j]);
                    #endif
                    
                    enable_udma(udev, &usb_data[0], target);
                }
                usb_release_interface(udev,0);
            }
            
            usb_close(udev);        
    	}
    }
    return ret;
}

int main(int argc, char *argv[])
{
  struct usb_bus *bus;
  int ret, isd300_found = FIND_ERROR;

  printf("\nUtility to enable UDMA in ISD-300 USB-ATA bridge");
  printf("\n================================================\n");
    
  usb_init();
  usb_set_debug(0);

  usb_find_busses();
  usb_find_devices();

  for (bus = usb_get_busses(); bus; bus = bus->next) 
  {
        if (bus->root_dev)
        {
            ret=find_and_modify_isd300(bus->root_dev);
            if (ret != FIND_ERROR) isd300_found = ret;
            if (ret == FIND_ABORT) break;
        }
  }

  if (isd300_found == FIND_ERROR) printf("Compatible device not found\n");
  
  if (isd300_found != FIND_ABORT)
  {
      printf("\nPress a key to exit\n");
      getch();
  }
  
  return 0;
}

